package frc.robot.subsystems;

import com.revrobotics.CANSparkMax;
import com.revrobotics.RelativeEncoder;
import com.revrobotics.CANSparkBase.IdleMode;
import com.revrobotics.CANSparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ArmFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.IntakeConstants;

public class Intake extends SubsystemBase{
    private PIDController intakeAnglePID;

    private CANSparkMax intakeRoller;
    private CANSparkMax intakePivot;

    private RelativeEncoder intakeEncoder;

    private ArmFeedforward intakeFeedForward;

    public Intake(){
        intakeRoller = new CANSparkMax(IntakeConstants.kIntakeRollerID, MotorType.kBrushless);
        
        intakePivot = new CANSparkMax(IntakeConstants.kIntakePivotID, MotorType.kBrushless);
        intakePivot.setIdleMode(IdleMode.kBrake);
        intakePivot.setSmartCurrentLimit(IntakeConstants.kPivotCurrentLimit);
        intakePivot.burnFlash();

        intakeFeedForward = new ArmFeedforward(
            IntakeConstants.kSFeedForward, 
            IntakeConstants.kGFeedForward, 
            IntakeConstants.kVFeedForward
        );

        intakeEncoder = intakePivot.getEncoder();
        intakeEncoder.setPosition(IntakeConstants.kStartingAngle);
        intakeEncoder.setPositionConversionFactor(IntakeConstants.kIntakePivotConversionFactor);

        intakeAnglePID = new PIDController(
            IntakeConstants.kPIntake, 
            IntakeConstants.kIIntake, 
            IntakeConstants.kDIntake
        );

    }

    public Command intakeDownCommand() {
        return run(() -> {
            intakeRoller.set(1.0);
        
            intakePivot.setVoltage(
                intakeAnglePID.calculate(
                    intakeEncoder.getPosition(), 
                    IntakeConstants.kDownAngle
                ) + intakeFeedForward.calculate(
                    IntakeConstants.kDownAngle, 
                    intakeEncoder.getVelocity()
                )
            );
        });
    }

    public Command climbingStateCommand() {
        return run(() -> {
            intakeRoller.set(0.0);

            intakePivot.setVoltage(
                intakeAnglePID.calculate(
                    intakeEncoder.getPosition(), 
                    IntakeConstants.kDownAngle
                ) + intakeFeedForward.calculate(
                    IntakeConstants.kDownAngle, 
                    intakeEncoder.getVelocity()
                )
            );
        });
    }

    public Command intakeUpCommand() {
        return run(() -> {
            intakeRoller.set(0.0);

            intakePivot.setVoltage(
                intakeAnglePID.calculate(
                    intakeEncoder.getPosition(), 
                    IntakeConstants.kUpAngle
                ) + intakeFeedForward.calculate(
                    IntakeConstants.kUpAngle, 
                    intakeEncoder.getVelocity()
                )
            );
        });
    }
}
